@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2020 SAP SE. All rights reserved.
+ * Copyright (c) 2020, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2020, 2021 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -33,6 +33,7 @@
 #include "memory/metaspace/metaspaceSettings.hpp"
 #include "memory/metaspace/virtualSpaceNode.hpp"
 #include "runtime/mutexLocker.hpp"
+#include "sanitizers/address.hpp"
 #include "utilities/debug.hpp"
 //#define LOG_PLEASE
 #include "metaspaceGtestCommon.hpp"
@@ -80,7 +81,7 @@ class VirtualSpaceNodeTest {
 
   void lock_and_verify_node() {
 #ifdef ASSERT
-    MutexLocker fcl(MetaspaceExpand_lock, Mutex::_no_safepoint_check_flag);
+    MutexLocker fcl(Metaspace_lock, Mutex::_no_safepoint_check_flag);
     _node->verify_locked();
 #endif
   }
@@ -92,7 +93,7 @@ class VirtualSpaceNodeTest {
     const bool node_is_full = _node->used_words() == _node->word_size();
     Metachunk* c = NULL;
     {
-      MutexLocker fcl(MetaspaceExpand_lock, Mutex::_no_safepoint_check_flag);
+      MutexLocker fcl(Metaspace_lock, Mutex::_no_safepoint_check_flag);
       c = _node->allocate_root_chunk();
     }
 
@@ -218,7 +219,7 @@ class VirtualSpaceNodeTest {
 
     // Split...
     {
-      MutexLocker fcl(MetaspaceExpand_lock, Mutex::_no_safepoint_check_flag);
+      MutexLocker fcl(Metaspace_lock, Mutex::_no_safepoint_check_flag);
       _node->split(target_level, c, freelist);
     }
 
@@ -259,7 +260,7 @@ class VirtualSpaceNodeTest {
 
     Metachunk* result = NULL;
     {
-      MutexLocker fcl(MetaspaceExpand_lock, Mutex::_no_safepoint_check_flag);
+      MutexLocker fcl(Metaspace_lock, Mutex::_no_safepoint_check_flag);
       result = _node->merge(c, freelist);
     }
     EXPECT_NOT_NULL(result);
@@ -297,7 +298,7 @@ public:
     _commit_limit(commit_limit)
   {
     {
-      MutexLocker fcl(MetaspaceExpand_lock, Mutex::_no_safepoint_check_flag);
+      MutexLocker fcl(Metaspace_lock, Mutex::_no_safepoint_check_flag);
       _node = VirtualSpaceNode::create_node(vs_word_size, &_commit_limiter,
                                             &_counter_reserved_words, &_counter_committed_words);
       EXPECT_EQ(_node->word_size(), vs_word_size);
@@ -308,7 +309,7 @@ public:
 
   ~VirtualSpaceNodeTest() {
     {
-      MutexLocker fcl(MetaspaceExpand_lock, Mutex::_no_safepoint_check_flag);
+      MutexLocker fcl(Metaspace_lock, Mutex::_no_safepoint_check_flag);
       delete _node;
     }
     // After the node is deleted, counters should be back to zero
@@ -354,7 +355,7 @@ public:
     TestMap testmap(c->word_size());
     assert(testmap.get_num_set() == 0, "Sanity");
 
-    for (int run = 0; run < 1000; run++) {
+    for (int run = 0; run < 750; run++) {
 
       const size_t committed_words_before = testmap.get_num_set();
       ASSERT_EQ(_commit_limiter.committed_words(), committed_words_before);
@@ -373,12 +374,14 @@ public:
 
         bool rc = false;
         {
-          MutexLocker fcl(MetaspaceExpand_lock, Mutex::_no_safepoint_check_flag);
+          MutexLocker fcl(Metaspace_lock, Mutex::_no_safepoint_check_flag);
           rc = _node->ensure_range_is_committed(c->base() + r.start(), r.size());
         }
 
         // Test-zap
+        ASAN_UNPOISON_MEMORY_REGION(c->base() + r.start(), r.size() * BytesPerWord);
         zap_range(c->base() + r.start(), r.size());
+        ASAN_POISON_MEMORY_REGION(c->base() + r.start(), r.size() * BytesPerWord);
 
         // We should never reach commit limit since it is as large as the whole area.
         ASSERT_TRUE(rc);
@@ -390,7 +393,7 @@ public:
         //LOG("u " SIZE_FORMAT "," SIZE_FORMAT, r.start(), r.end());
 
         {
-          MutexLocker fcl(MetaspaceExpand_lock, Mutex::_no_safepoint_check_flag);
+          MutexLocker fcl(Metaspace_lock, Mutex::_no_safepoint_check_flag);
           _node->uncommit_range(c->base() + r.start(), r.size());
         }
 
@@ -422,7 +425,7 @@ public:
 
     assert(_commit_limit >= _vs_word_size, "No commit limit here pls");
 
-    // Allocate a root chunk and commit a random part of it. Then repeatedly split
+    // Allocate a root chunk and commit a part of it. Then repeatedly split
     // it and merge it back together; observe the committed regions of the split chunks.
 
     Metachunk* c = alloc_root_chunk();
@@ -491,9 +494,9 @@ public:
 
 TEST_VM(metaspace, virtual_space_node_test_basics) {
 
-  MutexLocker fcl(MetaspaceExpand_lock, Mutex::_no_safepoint_check_flag);
+  MutexLocker fcl(Metaspace_lock, Mutex::_no_safepoint_check_flag);
 
-  const size_t word_size = metaspace::chunklevel::MAX_CHUNK_WORD_SIZE * 10;
+  const size_t word_size = metaspace::chunklevel::MAX_CHUNK_WORD_SIZE * 3;
 
   SizeCounter scomm;
   SizeCounter sres;
@@ -510,7 +513,9 @@ TEST_VM(metaspace, virtual_space_node_test_basics) {
   ASSERT_EQ(node->committed_words(), word_size);
   ASSERT_EQ(node->committed_words(), scomm.get());
   DEBUG_ONLY(node->verify_locked();)
+  ASAN_UNPOISON_MEMORY_REGION(node->base(), node->word_size() * BytesPerWord);
   zap_range(node->base(), node->word_size());
+  ASAN_POISON_MEMORY_REGION(node->base(), node->word_size() * BytesPerWord);
 
   node->uncommit_range(node->base(), node->word_size());
   ASSERT_EQ(node->committed_words(), (size_t)0);
@@ -524,7 +529,9 @@ TEST_VM(metaspace, virtual_space_node_test_basics) {
     ASSERT_EQ(node->committed_words(), i * Settings::commit_granule_words());
     ASSERT_EQ(node->committed_words(), scomm.get());
     DEBUG_ONLY(node->verify_locked();)
+    ASAN_UNPOISON_MEMORY_REGION(node->base(), i * Settings::commit_granule_words() * BytesPerWord);
     zap_range(node->base(), i * Settings::commit_granule_words());
+    ASAN_POISON_MEMORY_REGION(node->base(), i * Settings::commit_granule_words() * BytesPerWord);
   }
 
   node->uncommit_range(node->base(), node->word_size());
@@ -532,6 +539,9 @@ TEST_VM(metaspace, virtual_space_node_test_basics) {
   ASSERT_EQ(node->committed_words(), scomm.get());
   DEBUG_ONLY(node->verify_locked();)
 
+  delete node;
+  ASSERT_EQ(scomm.get(), (size_t)0);
+  ASSERT_EQ(sres.get(), (size_t)0);
 }
 
 // Note: we unfortunately need TEST_VM even though the system tested
@@ -552,15 +562,10 @@ TEST_VM(metaspace, virtual_space_node_test_2) {
 }
 
 TEST_VM(metaspace, virtual_space_node_test_3) {
-  double d = os::elapsedTime();
   // Test committing uncommitting arbitrary ranges
-  for (int run = 0; run < 100; run++) {
-    VirtualSpaceNodeTest test(metaspace::chunklevel::MAX_CHUNK_WORD_SIZE,
-        metaspace::chunklevel::MAX_CHUNK_WORD_SIZE);
-    test.test_split_and_merge_chunks();
-  }
-  double d2 = os::elapsedTime();
-  LOG("%f", (d2-d));
+  VirtualSpaceNodeTest test(metaspace::chunklevel::MAX_CHUNK_WORD_SIZE,
+      metaspace::chunklevel::MAX_CHUNK_WORD_SIZE);
+  test.test_split_and_merge_chunks();
 }
 
 TEST_VM(metaspace, virtual_space_node_test_4) {
@@ -580,13 +585,13 @@ TEST_VM(metaspace, virtual_space_node_test_5) {
 TEST_VM(metaspace, virtual_space_node_test_7) {
   // Test large allocation and freeing.
   {
-    VirtualSpaceNodeTest test(metaspace::chunklevel::MAX_CHUNK_WORD_SIZE * 100,
-        metaspace::chunklevel::MAX_CHUNK_WORD_SIZE * 100);
+    VirtualSpaceNodeTest test(metaspace::chunklevel::MAX_CHUNK_WORD_SIZE * 25,
+        metaspace::chunklevel::MAX_CHUNK_WORD_SIZE * 25);
     test.test_exhaust_node();
   }
   {
-    VirtualSpaceNodeTest test(metaspace::chunklevel::MAX_CHUNK_WORD_SIZE * 100,
-        metaspace::chunklevel::MAX_CHUNK_WORD_SIZE * 100);
+    VirtualSpaceNodeTest test(metaspace::chunklevel::MAX_CHUNK_WORD_SIZE * 25,
+        metaspace::chunklevel::MAX_CHUNK_WORD_SIZE * 25);
     test.test_exhaust_node();
   }
 
